@@ -11,9 +11,16 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: bi_funct.c,v 1.129 2024/08/15 00:39:17 tom Exp $
+ * $MawkId: bi_funct.c,v 1.132 2024/08/26 08:11:02 tom Exp $
  */
 
+#define Visible_ARRAY
+#define Visible_BI_REC
+#define Visible_CELL
+#define Visible_RE_DATA
+#define Visible_STRING
+#define Visible_SYMTAB
+
 #include <mawk.h>
 #include <bi_funct.h>
 #include <bi_vars.h>
@@ -56,7 +63,7 @@ const BI_REC bi_funct[] =
 
    { "index",    bi_index,    2, 2 },
    { "substr",   bi_substr,   2, 3 },
-   { "sprintf",  bi_sprintf,  1, 255 },
+   { "sprintf",  bi_sprintf,  1, MAX_ARGS },
    { "sin",      bi_sin,      1, 1 },
    { "cos",      bi_cos,      1, 1 },
    { "atan2",    bi_atan2,    2, 2 },
