@@ -1,6 +1,6 @@
 /********************************************
 init.c
-copyright 2008-2020,2021, Thomas E. Dickey
+copyright 2008-2023,2024, Thomas E. Dickey
 copyright 1991-1994,1995, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,10 +11,16 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: init.c,v 1.74 2021/05/29 00:00:53 tom Exp $
+ * $MawkId: init.c,v 1.81 2024/08/25 18:27:39 tom Exp $
  */
 
-/* init.c */
+#define Visible_ARRAY
+#define Visible_BI_REC
+#define Visible_CELL
+#define Visible_PFILE
+#define Visible_STRING
+#define Visible_SYMTAB
+
 #include <mawk.h>
 #include <code.h>
 #include <memory.h>
@@ -108,7 +114,7 @@ no_program(void)
     mawk_exit(0);
 }
 
-static void
+static GCC_NORETURN void
 usage(FILE *fp)
 {
     static const char msg[][80] =
@@ -134,7 +140,6 @@ usage(FILE *fp)
 #if USE_BINMODE
 	"    -W binmode",
 #endif
-	"    -W compat        pre-POSIX 2001.",
 	"    -W dump          show assembler-like listing of program and exit.",
 	"    -W help          show this message and exit.",
 	"    -W interactive   set unbuffered output, line-buffered input.",
@@ -142,6 +147,7 @@ usage(FILE *fp)
 	"    -W posix         stricter POSIX checking.",
 	"    -W random=number set initial random seed.",
 	"    -W sprintf=number adjust size of sprintf buffer.",
+	"    -W traditional   pre-POSIX 2001.",
 	"    -W usage         show this message and exit.",
     };
     size_t n;
@@ -301,7 +307,7 @@ numeric_option(const char *source)
 {
     char *next = NULL;
     long result = strtol(source, &next, 0);
-    if (next == source || next == NULL || *next != '\0') {
+    if (next == source || next == NULL || (*next != '\0' && *next != ',')) {
 	errmsg(0, "invalid numeric option: \"%s\"", source);
 	mawk_exit(2);
     }
@@ -349,7 +355,8 @@ handle_w_opt(W_OPTIONS code, int glue, char *option, char **value)
     switch (code) {
     case W_VERSION:
 	print_version(stdout);
-	break;
+	/* NOTREACHED */
+
 #if USE_BINMODE
     case W_BINMODE:
 	wantArg = 1;
@@ -437,7 +444,7 @@ handle_w_opt(W_OPTIONS code, int glue, char *option, char **value)
     case W_USAGE:
 	usage(stdout);
 	/* NOTREACHED */
-	break;
+
     case W_UNKNOWN:
 	errmsg(0, "vacuous option: -W \"%s\"", option);
 	break;
@@ -451,18 +458,22 @@ handle_w_opt(W_OPTIONS code, int glue, char *option, char **value)
 	    mawk_exit(2);
 	}
 	optNext = skipValue(optNext);
-    } else {
+    } else if (glue) {
 	while (glue) {
 	    errmsg(0, "unexpected option value \"%s\"", option);
 	    optNext = skipValue(optNext);
 	    glue = haveValue(optNext);
 	}
+    } else if (*value == NULL) {
+	optNext = skipValue(option);
+	if (*optNext == ',')
+	    ++optNext;
     }
     *value = optNext;
     return result;
 }
 
-static void
+static GCC_NORETURN void
 bad_option(const char *s)
 {
     errmsg(0, "not an option: %s", s);
@@ -480,7 +491,7 @@ allow_long_options(char *arg, W_OPTIONS seen)
 	default:
 	case 'e':		/* error */
 	    bad_option(arg);
-	    break;
+	    /* NOTREACHED */
 	case 'w':		/* warn */
 	    errmsg(0, "ignored option: %s", arg);
 	    break;
@@ -572,6 +583,7 @@ process_cmdline(int argc, char **argv)
 		optNext = optValue;
 	    } else {
 		bad_option(curArg);
+		/* NOTREACHED */
 	    }
 	    continue;
 	}
@@ -583,6 +595,7 @@ process_cmdline(int argc, char **argv)
 		    mawk_exit(2);
 		}
 		bad_option(curArg);
+		/* NOTREACHED */
 	    }
 
 	    optArg = argv[i + 1];
@@ -595,7 +608,11 @@ process_cmdline(int argc, char **argv)
 	switch (curArg[1]) {
 
 	case 'W':
-	    for (j = 0; j < (int) strlen(optArg); j = (int) (optNext - optArg)) {
+	    for (j = 0;
+		 j < (int) strlen(optArg);
+		 j = (optNext
+		      ? (int) (optNext - optArg)
+		      : (int) strlen(optArg))) {
 		char *name = optArg + j;
 		int args = 0;
 		W_OPTIONS code = parse_w_opt(name, &optNext, &args);
@@ -640,8 +657,9 @@ process_cmdline(int argc, char **argv)
 	case '-':
 	    if (curArg[2] != 0) {
 		bad_option(curArg);
+		/* NOTREACHED */
 	    }
-	    i++;
+	    curArg = argv[++i];
 	    goto no_more_opts;
 
 	case 'f':
@@ -657,6 +675,7 @@ process_cmdline(int argc, char **argv)
 
 	default:
 	    bad_option(curArg);
+	    /* NOTREACHED */
 	}
     }
 
